/*********************************************************************
 * key_de.h - Keyboard mapping VIC-20 keyboard to German keyboard    *
 * setting on the PC side                                            *
 *********************************************************************
 * vic20_kbd is free software                                        *
 *                                                                   *
 * vic20_kbd is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.              *
 *********************************************************************
 * This keyboard mapping supports joystick numpad emulation for the  *
 * VICE emulator. Be shure to have NumLock turned on. Here is a      *
 * little joystick test program for the VIC-20:                      *
 *                                                                   *
10 dim js(2,2):poke371
39,0:dd=37154:pa=37137
:pb=37152
20 fori=0to2:forj=0to2
:readjs(j,i):nextj,i
25 data-23,-22,-21,-1,
0,1,21,22,23
30 gosub9000:printjs(x
+1,y+1):printfr*99:got
o30
9000 pokedd,127:s3=-((
peek(pb)and128)=0):pok
edd,255
9010 p=peek(pa):s1=-((
pand8)=0):s2=((pand16)
=0):s0=((pand4)=0)
9020 fr=-((pand32)=0):
x=s2+s3:y=s0+s1:return
 * Just paste it int VICE                                           *
 ********************************************************************/
 
#ifndef KEYMAP_H
#define KEYMAP_H

  /* Other keys that need special handling -
     These are looked up in the table spec_keys because they do not
     generate the same scan-code in the shifted and unshifted state,
     and some may need to alter the shift-state to generate the
     correct character code on the PC */
enum specialkeys {
  SPC_larr = 1,
  SPC_3,
  SPC_7,
  SPC_0,
  SPC_plus,
  SPC_minus,
//  SPC_pound,
//  SPC_home,
//  SPC_del,
  SPC_at,
  SPC_ast,
  SPC_col,
  SPC_smcol,
  SPC_equal,
  SPC_comma,
  SPC_dot,
  SPC_slash,
//  SPC_crsrud,
//  SPC_crsrlr,
//  SPC_F1,
//  SPC_F3,
//  SPC_F5,
//  SPC_F7
};

/*
 * The keymap for the Commodore VIC-20 keyboard
 * to a PC with german keyboard mapping
 */
const prog_uchar keymap[] PROGMEM = { // German keymap
    HKEY_1, SPC_3, HKEY_5, SPC_7, HKEY_9, SPC_plus, /* SPC_pound */ HKEY_ins, /* SPC_del */ HKEY_bckspc, // row0
    SPC_larr, HKEY_W, HKEY_R, HKEY_Z, HKEY_I, HKEY_P, SPC_ast, HKEY_enter, // row1
    HKEY_tab, HKEY_A, HKEY_D, HKEY_G, HKEY_J, HKEY_L, /* HKEY_ping */ SPC_smcol, /* SPC_crsrlr */ HKEY_rarr, // row2
    HKEY_esc, KEY_LEFT_SHIFT, HKEY_X, HKEY_V, HKEY_N, SPC_comma, SPC_slash, HKEY_darr, // row3
    HKEY_spc, HKEY_Y, HKEY_C, HKEY_B, HKEY_M, SPC_dot, KEY_RIGHT_SHIFT, HKEY_F1, // row4
    KEY_LEFT_CTRL, HKEY_S, HKEY_F, HKEY_H, HKEY_K, /* HKEY_smcol */ SPC_col, SPC_equal, HKEY_F3, // row5
    HKEY_Q, HKEY_E, HKEY_T, HKEY_U, HKEY_O, /* HKEY_lbr */ SPC_at, /* HKEY_rbr */ HKEY_del, HKEY_F5, // row6
    HKEY_2, HKEY_4, HKEY_6, HKEY_8, SPC_0, SPC_minus, /* SPC_home */ HKEY_home, HKEY_F7, // row7
    HKEY_pgup, 0, 0, HKEY_KP0, HKEY_KP8, HKEY_KP2, HKEY_KP6, HKEY_KP4 // Imaginary row8 for restore and joystick
    //               FIRE      FORWARD   BACK      RIGHT     LEFT        
};

/*
 * Special keys that need to generate different scan-codes for unshifted
 * and shifted states, or that need to alter the modifier keys. 
 * Since the LGUI and RGUI bits are not used, these signify that the
 * left and right shift states should be deleted from report, so
 *   0x00 means do not alter shift states
 *   0x02 means set L_SHIFT
 *   0x88 means clear both shift flags
 *   0xC8 means clear both shifts and set R_ALT
 */
const prog_uchar spec_keys[] PROGMEM = {
  HKEY_slash,   0x02, HKEY_slash,   0x00, // SPC_larr - left arrow is _
  HKEY_3,       0x00, HKEY_hash,    0x88, // SPC_3 - shift-3 is #
  HKEY_7,       0x00, HKEY_hash,    0x00, // SPC_7 - shift-7 is ´
  HKEY_0,       0x00, HKEY_0,       0x88, // SPC_0 - shift-0 is still 0
  HKEY_rbr,     0x00, HKEY_KPplus,  0x00, // SPC_plus 
  HKEY_slash,   0x00, HKEY_KPminus, 0x00, // SPC_minus - "-" and KP "-"
//  HKEY_dot,     0x8A, HKEY_dot,     0x8A, // SPC_pound - ":"
//  HKEY_home,    0x80, HKEY_end,     0x80, // SPC_home - home and end
//  HKEY_bckspc,  0x00, HKEY_del,     0x88, // SPC_del - backspace and delete
  HKEY_Q,       0x40, HKEY_end,     0x00, // SPC_at - Alt-Q
//  HKEY_rbr,     0x02, HKEY_KPast,   0x00, // SPC_ast - "*" (Asterix)
  HKEY_KPast,   0x00, HKEY_KPast,   0x00, // SPC_ast - "*" (Asterix)
  HKEY_dot,     0x02, HKEY_8,       0xC8, // SPC_col - ":" and "["
  HKEY_comma,   0x02, HKEY_9,       0xC8, // SPC_smcol - ";" and "]"
  HKEY_0,       0x02, HKEY_0,       0x00, // SPC_equal - shift-0 is =
  HKEY_comma,   0x80, HKEY_Euro2,   0x88, // SPC_comma - "," and "<"
  HKEY_dot,     0x80, HKEY_Euro2,   0x8A, // SPC_dot - "." and ">"
  HKEY_7,       0x02, HKEY_minus,   0x00, // SPC_slash - "/" and "?"
//  HKEY_darr,    0x80, HKEY_uarr,    0x80, // SPC_crsrud - cursor down/up
//  HKEY_rarr,    0x80, HKEY_larr,    0x80, // SPC_crsrlr - cursor right/left
//  HKEY_F1,      0x80, HKEY_F2,      0x80, // SPC_F1 - F1 and F2
//  HKEY_F3,      0x80, HKEY_F4,      0x80, // SPC_F3 - F3 and F4
//  HKEY_F5,      0x80, HKEY_F6,      0x80, // SPC_F5 - F5 and F6
//  HKEY_F7,      0x80, HKEY_F8,      0x80, // SPC_F7 - F7 and F8
};
#endif


