/*********************************************************************
 * Original file: key_us_us.h                                        *
 * key_us.h - Keyboard mapping VIC-20 keyboard to American           *
 * keyboard setting on the PC side                                   *
 *********************************************************************
 * vic20_kbd is free software; because it does not (like the         *
 * original) use the V-USB software, there is no need to put it      *
 * under the OBDEV license.                                          *
 *                                                                   *
 * vic20_kbd is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.              *
 *********************************************************************/
#ifndef KEYMAP_H
#define KEYMAP_H

  /* Other keys that need special handling -
     These are looked up in the table spec_keys because they do not
     generate the same scan-code in the shifted and unshifted state,
     and some may need to alter the shift-state to generate the
     correct character code on the PC */
enum specialkeys {
  SPC_2 = 1,
  SPC_6,
  SPC_7,
  SPC_8,
  SPC_9,
  SPC_0,
  SPC_plus,
  SPC_minus,
  SPC_pound,
  SPC_home,
  SPC_del,
  SPC_ast,
  SPC_equal,
  SPC_crsrud,
  SPC_crsrlr,
  SPC_F1,
  SPC_F3,
  SPC_F5,
  SPC_F7,
  SPC_hat,
  SPC_colon,
  SPC_smcol,
  SPC_at
};

/*
 * The keymap for the Commodore VIC-20 keyboard
 * to a PC with american keyboard mapping
 */
const prog_uchar keymap[] PROGMEM = { // American keymap
    HKEY_1, HKEY_3, HKEY_5, SPC_7, SPC_9, SPC_plus, SPC_pound, SPC_del, // row0
    HKEY_esc, HKEY_W, HKEY_R, HKEY_Y, HKEY_I, HKEY_P, SPC_ast, HKEY_enter, // row1
    KEY_LEFT_CTRL, HKEY_A, HKEY_D, HKEY_G, HKEY_J, HKEY_L, SPC_smcol, SPC_crsrlr, // row2
    KEY_RIGHT_ALT, KEY_LEFT_SHIFT, HKEY_X, HKEY_V, HKEY_N, HKEY_comma, HKEY_slash, SPC_crsrud, // row3
    HKEY_spc, HKEY_Z, HKEY_C, HKEY_B, HKEY_M, HKEY_dot, KEY_RIGHT_SHIFT, SPC_F1, // row4
    KEY_LEFT_ALT, HKEY_S, HKEY_F, HKEY_H, HKEY_K, SPC_colon, SPC_equal, SPC_F3, // row5
    HKEY_Q, HKEY_E, HKEY_T, HKEY_U, HKEY_O, SPC_at, SPC_hat, SPC_F5, // row6
    SPC_2, HKEY_4, SPC_6, SPC_8, SPC_0, SPC_minus, SPC_home, SPC_F7, // row7
    KEY_RIGHT_CTRL, 0, 0, HKEY_KP0, HKEY_KP8, HKEY_KP2, HKEY_KP6, HKEY_KP4 // Imaginary row8 for restore and joystick
    //                    FIRE      FORWARD   BACK      RIGHT     LEFT        
  };

/* Special keys that need to generate different scan-codes for unshifted
   and shifted states, or that need to alter the modifier keys. 
   Since the LGUI and RGUI bits are not used, these signify that the
   left and right shift states should be deleted from report, so
     0x88 means clear both shift flags
     0x00 means do not alter shift states
     0xC8 means clear both shifts and set L_ALT */
const prog_uchar spec_keys[] PROGMEM = {
   HKEY_2,       0x00, HKEY_ping,    0x00, // SPC_2 - shift-2 is "
   HKEY_6,       0x00, HKEY_7,       0x00, // SPC_6 - shift-6 is &
   HKEY_7,       0x00, HKEY_ping,    0x88, // SPC_7 - shift-7 is '
   HKEY_8,       0x00, HKEY_9,       0x00, // SPC_8 - shift-8 is (
   HKEY_9,       0x00, HKEY_0,       0x00, // SPC_9 - shift-9 is )
   HKEY_0,       0x00, HKEY_0,       0x88, // SPC_0 - shift-0 is 0
   HKEY_equal,   0x02, HKEY_equal,   0x8A, // SPC_plus 
   HKEY_minus,   0x00, HKEY_minus,   0x88, // SPC_minus - "-" and "-"
   HKEY_grave,   0x02, HKEY_grave,   0x8A, // SPC_pound - "~"
   HKEY_home,    0x80, HKEY_end,     0x80, // SPC_home - home and end
   HKEY_bckspc,  0x00, HKEY_del,     0x88, // SPC_del - backspace and delete
   HKEY_8,       0x02, HKEY_8,       0x02, // SPC_ast - "*" (Asterix)
   HKEY_equal,   0x00, HKEY_equal,   0x88, // SPC_equal - "="
   HKEY_darr,    0x80, HKEY_uarr,    0x80, // SPC_crsrud - cursor down/up
   HKEY_rarr,    0x80, HKEY_larr,    0x80, // SPC_crsrlr - cursor right/left
   HKEY_F1,      0x80, HKEY_F2,      0x80, // SPC_F1 - F1 and F2
   HKEY_F3,      0x80, HKEY_F4,      0x80, // SPC_F3 - F3 and F4
   HKEY_F5,      0x80, HKEY_F6,      0x80, // SPC_F5 - F5 and F6
   HKEY_F7,      0x80, HKEY_F8,      0x80, // SPC_F7 - F7 and F8
   HKEY_6,       0x02, HKEY_6,       0x00, // SPC_hat - "^"
   HKEY_smcol,   0x02, HKEY_lbr,     0x88, // SPC_colon - : and [
   HKEY_smcol,   0x00, HKEY_rbr,     0x88, // SPC_smcol - ; and ]
   HKEY_2,       0x8A, HKEY_2,       0x8A, // SPC_at - @
};
#endif


